/*
 * Copyright 2011-16 Fraunhofer ISE, energy & meteo Systems GmbH and other contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.openmuc.openiec61850;

import org.openmuc.jasn1.ber.types.BerBitString;
import org.openmuc.openiec61850.internal.mms.asn1.Data;
import org.openmuc.openiec61850.internal.mms.asn1.Integer32;
import org.openmuc.openiec61850.internal.mms.asn1.TypeDescription;

abstract public class BdaBitString extends BasicDataAttribute {

    byte[] value;
    final int maxNumBits;

    public BdaBitString(ObjectReference objectReference, Fc fc, String sAddr, int maxNumBits, boolean dchg,
            boolean dupd) {
        super(objectReference, fc, sAddr, dchg, dupd);
        this.maxNumBits = maxNumBits;
    }

    public byte[] getValue() {
        return value;
    }

    public void setValue(byte[] value) {
        this.value = value;
    }

    @Override
    void setValueFrom(BasicDataAttribute bda) {
        byte[] srcValue = ((BdaBitString) bda).getValue();
        if (value.length != srcValue.length) {
            value = new byte[srcValue.length];
        }
        System.arraycopy(srcValue, 0, value, 0, srcValue.length);
    }

    public int getMaxNumBits() {
        return maxNumBits;
    }

    /**
     * Initializes BIT_STRING with all zeros
     */
    @Override
    public void setDefault() {
        value = new byte[(maxNumBits / 8 + (((maxNumBits % 8) > 0) ? 1 : 0))];
    }

    @Override
    Data getMmsDataObj() {
        return new Data(null, null, null, new BerBitString(value, maxNumBits), null, null, null, null, null, null, null,
                null);
    }

    @Override
    void setValueFromMmsDataObj(Data data) throws ServiceError {
        if (data.bitString == null) {
            throw new ServiceError(ServiceError.TYPE_CONFLICT, "expected type: bit_string");
        }
        if (data.bitString.numBits > maxNumBits) {
            throw new ServiceError(ServiceError.TYPE_CONFLICT, objectReference
                    + ": bit_string is bigger than maxNumBits: " + data.bitString.numBits + ">" + maxNumBits);
        }
        value = data.bitString.value;
    }

    @Override
    TypeDescription getMmsTypeSpec() {
        return new TypeDescription(null, null, null, new Integer32(maxNumBits * -1), null, null, null, null, null, null,
                null, null);
    }

    @Override
    public String toString() {
        return getReference().toString() + ": " + HexConverter.toHexString(value);
    }

}
